"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const common_1 = require("./common");
class StaticFilesystemCreator {
    constructor() {
        this.index = new Array();
        this.intBuffer = Buffer.alloc(common_1.INTSIZE);
    }
    async addFolder(sourceFolder, targetFolder) {
        await this.getFileNames(sourceFolder, targetFolder);
    }
    get headerLength() {
        let size = common_1.INTSIZE; // start of data 
        // put hash size in header
        this.hashBuffer = Buffer.from(this.hash, 'utf-8');
        size += common_1.INTSIZE;
        size += this.hashBuffer.byteLength;
        for (const each in this.index) {
            size += common_1.INTSIZE; // name size
            size += common_1.INTSIZE; // data size
            const filenameBuffer = Buffer.from(each, 'utf-8');
            this.index[each].filename = filenameBuffer;
            size += filenameBuffer.byteLength; // name itself.
        }
        size += common_1.INTSIZE; // trailing zero.
        return size;
    }
    writeInt(fd, value, position) {
        this.intBuffer.writeIntBE(value, 0, 6);
        return common_1.write(fd, this.intBuffer, 0, common_1.INTSIZE, position);
    }
    async write(outputPath, hash = common_1.calculateHash(this.index)) {
        this.hash = hash;
        let dataOffset = this.headerLength;
        const fd = await common_1.open(outputPath, "w");
        let headerPosition = await this.writeInt(fd, dataOffset);
        headerPosition += await this.writeInt(fd, this.hashBuffer.byteLength);
        headerPosition += await common_1.write(fd, this.hashBuffer, 0, this.hashBuffer.byteLength, headerPosition);
        const all = [];
        // start writing out the data
        for (const each in this.index) {
            const entry = this.index[each];
            const position = dataOffset;
            dataOffset += entry.size;
            const buf = await this.index[each].getBuffer();
            await common_1.write(fd, buf, 0, buf.length, position);
        }
        // finish writing all the buffers.
        await Promise.all(all);
        // write the header 
        for (const each in this.index) {
            const entry = this.index[each];
            headerPosition += await this.writeInt(fd, entry.filename.length, headerPosition);
            headerPosition += await this.writeInt(fd, entry.size, headerPosition);
            headerPosition += await common_1.write(fd, entry.filename, 0, entry.filename.length, headerPosition);
        }
        await common_1.close(fd);
        return hash;
    }
    async getFileNames(sourceFolder, targetFolder) {
        const files = await common_1.readdir(sourceFolder);
        const all = new Array();
        for (const file of files) {
            // compute the path names
            const sourcePath = `${sourceFolder}/${file}`;
            const targetPath = `${targetFolder}/${file}`;
            // is this a directory 
            const ss = await common_1.stat(sourcePath);
            if (ss.isDirectory()) {
                all.push(this.getFileNames(sourcePath, targetPath));
            }
            else {
                // it's a file. capture the details.
                this.index[targetPath] = {
                    size: ss.size,
                    getBuffer: () => common_1.readFile(sourcePath)
                };
            }
            ;
        }
        // wait for children to finish
        await Promise.all(all);
    }
}
exports.StaticFilesystemCreator = StaticFilesystemCreator;
//# sourceMappingURL=creator.js.map