"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
// backup original filesystem.
const original_fs = Object.assign({}, fs);
var MemberType;
(function (MemberType) {
    MemberType[MemberType["Constructor"] = 0] = "Constructor";
    MemberType[MemberType["Method"] = 1] = "Method";
    MemberType[MemberType["Property"] = 2] = "Property";
})(MemberType || (MemberType = {}));
const metadata = {
    StatWatcher: MemberType.Constructor,
    FSWatcher: MemberType.Constructor,
    ReadStream: MemberType.Constructor,
    WriteStream: MemberType.Constructor,
    ReadFileStream: MemberType.Constructor,
    WriteFileStream: MemberType.Constructor,
    Stats: MemberType.Constructor,
    constants: MemberType.Property,
    F_OK: MemberType.Property,
    R_OK: MemberType.Property,
    W_OK: MemberType.Property,
    X_OK: MemberType.Property,
};
function patchFilesystem(volume, original = fs) {
    // create a backup before modification
    const backup = Object.assign({}, original);
    // iterate over the filesystem and patch members
    for (const member of Object.getOwnPropertyNames(original)) {
        if (typeof volume[member] !== typeof original[member]) {
            continue;
        }
        switch (metadata[member]) {
            case MemberType.Constructor:
                // bind as a constuctor
                original[member] = volume[member].bind(null, volume);
                break;
            case MemberType.Property:
                // overwrite property
                original[member] = volume[member];
                break;
            default:
                // bind as a method
                original[member] = volume[member].bind(volume);
                break;
        }
    }
    // return a delegate to undo those changes.
    return () => patchFilesystem(fs, backup);
}
exports.patchFilesystem = patchFilesystem;
//# sourceMappingURL=patch-filesystem.js.map